@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2019 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2020 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,10 +54,9 @@ typedef struct ingest_info_struct
     long elements_per_profile;  /* Each profile is a series of elements, each element is a series of measurements for different wavelengths */
     long num_wavelengths;       /* The number of different spectra in 1 element */
     double *wavelengths;
-    double *limb_radiance_data_values;
-    double *wavelengths_in_conversion_factor_table;
-    double *conversion_factors_in_table;
-    uint8_t nr_wavelengths_in_conversion_factor_table;
+    uint8_t sensitivity_curve_size;
+    double *sensitivity_curve;
+    double *sensitivity_curve_wavelengths;
     int upper;
     int corrected;
 } ingest_info;
@@ -211,11 +210,7 @@ static int get_main_data(ingest_info *info, const char *datasetname, const char
         }
         double_data++;
     }
-    if (coda_cursor_goto_root(&cursor) != 0)
-    {
-        harp_set_error(HARP_ERROR_CODA, NULL);
-        return -1;
-    }
+
     return 0;
 }
 
@@ -249,11 +244,7 @@ static int get_wavelength_data(ingest_info *info, const char *datasetname, const
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    if (coda_cursor_goto_root(&cursor) != 0)
-    {
-        harp_set_error(HARP_ERROR_CODA, NULL);
-        return -1;
-    }
+
     return 0;
 }
 
@@ -322,11 +313,7 @@ static int get_spectral_data(ingest_info *info, const char *datasetname, const c
             return -1;
         }
     }
-    if (coda_cursor_goto_root(&cursor) != 0)
-    {
-        harp_set_error(HARP_ERROR_CODA, NULL);
-        return -1;
-    }
+
     return 0;
 }
 
@@ -377,8 +364,7 @@ static int get_illumination_condition(ingest_info *info, const char *datasetname
 }
 
 
-static int read_spectral_conversion_factors(ingest_info *info, char *datasetname, char *num_fieldname,
-                                            char *wavelength_fieldname, char *factor_fieldname)
+static int read_sensitivity_curve(ingest_info *info)
 {
     coda_cursor cursor;
 
@@ -387,7 +373,7 @@ static int read_spectral_conversion_factors(ingest_info *info, char *datasetname
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    if (coda_cursor_goto_record_field_by_name(&cursor, datasetname) != 0)
+    if (coda_cursor_goto_record_field_by_name(&cursor, "lim_occultation_data") != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
@@ -399,12 +385,12 @@ static int read_spectral_conversion_factors(ingest_info *info, char *datasetname
     }
 
     /* Read number of wavelengths in the lookup table */
-    if (coda_cursor_goto_record_field_by_name(&cursor, num_fieldname) != 0)
+    if (coda_cursor_goto_record_field_by_name(&cursor, "size_rad_sens_curve_limb") != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    if (coda_cursor_read_uint8(&cursor, &info->nr_wavelengths_in_conversion_factor_table) != 0)
+    if (coda_cursor_read_uint8(&cursor, &info->sensitivity_curve_size) != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
@@ -412,16 +398,14 @@ static int read_spectral_conversion_factors(ingest_info *info, char *datasetname
     coda_cursor_goto_parent(&cursor);
 
     /* Read wavelengths in the lookup table */
-    CHECKED_MALLOC(info->wavelengths_in_conversion_factor_table,
-                   sizeof(double) * info->nr_wavelengths_in_conversion_factor_table);
-    if (coda_cursor_goto_record_field_by_name(&cursor, wavelength_fieldname) != 0)
+    CHECKED_MALLOC(info->sensitivity_curve_wavelengths, sizeof(double) * info->sensitivity_curve_size);
+    if (coda_cursor_goto_record_field_by_name(&cursor, "abs_rad_sens_curve_limb") != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    if (coda_cursor_read_double_partial_array
-        (&cursor, 0L, (long)(info->nr_wavelengths_in_conversion_factor_table),
-         info->wavelengths_in_conversion_factor_table) != 0)
+    if (coda_cursor_read_double_partial_array(&cursor, 0L, (long)(info->sensitivity_curve_size),
+                                              info->sensitivity_curve_wavelengths) != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
@@ -429,66 +413,50 @@ static int read_spectral_conversion_factors(ingest_info *info, char *datasetname
     coda_cursor_goto_parent(&cursor);
 
     /* Read conversion factors in the lookup table */
-    CHECKED_MALLOC(info->conversion_factors_in_table, sizeof(double) * info->nr_wavelengths_in_conversion_factor_table);
-    if (coda_cursor_goto_record_field_by_name(&cursor, factor_fieldname) != 0)
+    CHECKED_MALLOC(info->sensitivity_curve, sizeof(double) * info->sensitivity_curve_size);
+    if (coda_cursor_goto_record_field_by_name(&cursor, "rad_sens_curve_limb") != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    if (coda_cursor_read_double_partial_array
-        (&cursor, 0L, (long)(info->nr_wavelengths_in_conversion_factor_table), info->conversion_factors_in_table) != 0)
+    if (coda_cursor_read_double_partial_array(&cursor, 0L, (long)(info->sensitivity_curve_size),
+                                              info->sensitivity_curve) != 0)
     {
         harp_set_error(HARP_ERROR_CODA, NULL);
         return -1;
     }
-    coda_cursor_goto_root(&cursor);
+
     return 0;
 }
 
-static void free_spectral_conversion_factors(ingest_info *info)
-{
-    if (info->wavelengths_in_conversion_factor_table != NULL)
-    {
-        free(info->wavelengths_in_conversion_factor_table);
-        info->wavelengths_in_conversion_factor_table = NULL;
-    }
-    if (info->conversion_factors_in_table != NULL)
-    {
-        free(info->conversion_factors_in_table);
-        info->conversion_factors_in_table = NULL;
-    }
-}
-
-static int spectral_conversion_factor(ingest_info *info, double wavelength, double *conversion_factor)
+static double spectral_conversion_factor(ingest_info *info, double wavelength)
 {
     uint8_t i;
 
-    if (wavelength < info->wavelengths_in_conversion_factor_table[0])
+    if (wavelength < info->sensitivity_curve_wavelengths[0])
     {
-        *conversion_factor = info->conversion_factors_in_table[0];
+        return info->sensitivity_curve[0];
     }
-    else if (wavelength >
-             info->wavelengths_in_conversion_factor_table[info->nr_wavelengths_in_conversion_factor_table - 1])
+    else if (wavelength > info->sensitivity_curve_wavelengths[info->sensitivity_curve_size - 1])
     {
-        *conversion_factor = info->conversion_factors_in_table[info->nr_wavelengths_in_conversion_factor_table - 1];
+        return info->sensitivity_curve[info->sensitivity_curve_size - 1];
     }
     else
     {
-        for (i = 1; i < info->nr_wavelengths_in_conversion_factor_table; i++)
+        for (i = 1; i < info->sensitivity_curve_size; i++)
         {
-            if ((wavelength > info->wavelengths_in_conversion_factor_table[i - 1]) &&
-                (wavelength < info->wavelengths_in_conversion_factor_table[i]))
+            if ((wavelength > info->sensitivity_curve_wavelengths[i - 1]) &&
+                (wavelength < info->sensitivity_curve_wavelengths[i]))
             {
-                *conversion_factor = info->conversion_factors_in_table[i - 1] +
-                    ((info->conversion_factors_in_table[i] - info->conversion_factors_in_table[i - 1]) *
-                     ((wavelength - info->wavelengths_in_conversion_factor_table[i - 1]) /
-                      (info->wavelengths_in_conversion_factor_table[i] -
-                       info->wavelengths_in_conversion_factor_table[i - 1])));
+                return info->sensitivity_curve[i - 1] +
+                    ((info->sensitivity_curve[i] - info->sensitivity_curve[i - 1]) *
+                     ((wavelength - info->sensitivity_curve_wavelengths[i - 1]) /
+                      (info->sensitivity_curve_wavelengths[i] - info->sensitivity_curve_wavelengths[i - 1])));
             }
         }
     }
 
-    return 0;
+    return harp_nan();
 }
 
 static void ingestion_done(void *user_data)
@@ -499,6 +467,14 @@ static void ingestion_done(void *user_data)
     {
         free(info->wavelengths);
     }
+    if (info->sensitivity_curve_wavelengths != NULL)
+    {
+        free(info->sensitivity_curve_wavelengths);
+    }
+    if (info->sensitivity_curve != NULL)
+    {
+        free(info->sensitivity_curve);
+    }
     free(info);
 }
 
@@ -534,8 +510,7 @@ static int read_lim_altitude(void *user_data, harp_array data)
 
 static int read_lim_spectral_photon_radiance(void *user_data, harp_array data)
 {
-    int retval;
-    double background_in_electrons, wavelength, conversion_factor;
+    double background_in_electrons;
     double *background_code_values, *background_offsets, *background_gains;
     double *double_data;
     ingest_info *info;
@@ -544,43 +519,26 @@ static int read_lim_spectral_photon_radiance(void *user_data, harp_array data)
     info = (ingest_info *)user_data;
 
     CHECKED_MALLOC(background_code_values, sizeof(double) * info->elements_per_profile * info->num_wavelengths);
-    if (info->corrected)
-    {
-        retval = get_spectral_data(info, "lim_mds", "up_low_back_corr", info->upper ? 0 : info->num_wavelengths,
-                                   background_code_values);
-    }
-    else
-    {
-        retval = get_spectral_data(info, "lim_mds", "up_low_back_no_corr", info->upper ? 0 : info->num_wavelengths,
-                                   background_code_values);
-    }
-    if (retval != 0)
+    if (get_spectral_data(info, "lim_mds", info->corrected ? "up_low_back_corr" : "up_low_back_no_corr",
+                          info->upper ? 0 : info->num_wavelengths, background_code_values) != 0)
     {
         free(background_code_values);
-        return retval;
+        return -1;
     }
 
     CHECKED_MALLOC(background_offsets, sizeof(double) * info->elements_per_profile);
-    retval = get_main_data((ingest_info *)user_data, "lim_ads", "off_back", IS_NO_ARRAY, background_offsets);
-    if (retval != 0)
+    if (get_main_data((ingest_info *)user_data, "lim_ads", "off_back", IS_NO_ARRAY, background_offsets) != 0)
     {
         free(background_offsets);
         free(background_code_values);
-        return retval;
+        return -1;
     }
     CHECKED_MALLOC(background_gains, sizeof(double) * info->elements_per_profile);
-    retval = get_main_data((ingest_info *)user_data, "lim_ads", "gain_back", IS_NO_ARRAY, background_gains);
-    if (retval != 0)
+    if (get_main_data((ingest_info *)user_data, "lim_ads", "gain_back", IS_NO_ARRAY, background_gains) != 0)
     {
         free(background_gains);
         free(background_offsets);
         free(background_code_values);
-        return retval;
-    }
-    if (read_spectral_conversion_factors(info, "lim_occultation_data", "size_rad_sens_curve_limb",
-                                         "abs_rad_sens_curve_limb", "rad_sens_curve_limb") != 0)
-    {
-        free_spectral_conversion_factors(info);
         return -1;
     }
 
@@ -595,65 +553,56 @@ static int read_lim_spectral_photon_radiance(void *user_data, harp_array data)
             background_in_electrons = background_offsets[profile_nr] +
                 background_code_values[profile_nr * info->num_wavelengths + wavelength_nr] /
                 background_gains[profile_nr];
-            wavelength = info->wavelengths[wavelength_nr];
             /* Convert the value in electrons to a physical unit according */
             /* to section 10.4.2.7.2 in the ENVISAT-GOMOS product          */
             /* specifications (ESA Doc Ref: PO-RS-MDA-GS-2009).            */
-            conversion_factor = 1.0;
-            if (spectral_conversion_factor(info, wavelength, &conversion_factor) == 0)
-            {
-                *double_data = background_in_electrons * conversion_factor;
-            }
-            else
-            {
-                *double_data = 0.0;
-            }
+            *double_data = background_in_electrons * spectral_conversion_factor(info, info->wavelengths[wavelength_nr]);
             double_data++;
         }
     }
 
-    free_spectral_conversion_factors(info);
     free(background_gains);
     free(background_offsets);
     free(background_code_values);
 
-    CHECKED_MALLOC(info->limb_radiance_data_values,
-                   sizeof(double) * info->elements_per_profile * info->num_wavelengths);
-    memcpy(info->limb_radiance_data_values, data.double_data,
-           sizeof(double) * info->elements_per_profile * info->num_wavelengths);
-
     return 0;
 }
 
 static int read_lim_spectral_photon_radiance_error(void *user_data, harp_array data)
 {
     ingest_info *info;
-    double *measured_data, *percentage;
+    harp_array measured_data;
     long i;
-    int retval;
 
     info = (ingest_info *)user_data;
-    retval = get_spectral_data(info, "lim_mds", "err_up_low_back_corr", info->upper ? 0 : info->num_wavelengths,
-                               data.double_data);
-    measured_data = info->limb_radiance_data_values;
-    percentage = data.double_data;
+
+    if (get_spectral_data(info, "lim_mds", "err_up_low_back_corr", info->upper ? 0 : info->num_wavelengths,
+                          data.double_data) != 0)
+    {
+        return -1;
+    }
+
+    CHECKED_MALLOC(measured_data.double_data, sizeof(double) * info->elements_per_profile * info->num_wavelengths);
+    if (read_lim_spectral_photon_radiance(user_data, measured_data) != 0)
+    {
+        free(measured_data.double_data);
+        return -1;
+    }
+
     for (i = 0; i < info->elements_per_profile * info->num_wavelengths; i++)
     {
         /* From percentage and measured datavalue, calculate uncertainty */
-        *percentage = fabs(0.01 * (*percentage) * (*measured_data));
-        measured_data++;
-        percentage++;
+        data.double_data[i] = fabs(0.01 * data.double_data[i] * measured_data.double_data[i]);
     }
-    free(info->limb_radiance_data_values);
-    return retval;
+
+    free(measured_data.double_data);
+
+    return 0;
 }
 
 static int read_lim_wavelength(void *user_data, harp_array data)
 {
-    int retval;
-
-    retval = get_wavelength_data((ingest_info *)user_data, "lim_nom_wav_assignment", "nom_wl", data.double_data);
-    return retval;
+    return get_wavelength_data((ingest_info *)user_data, "lim_nom_wav_assignment", "nom_wl", data.double_data);
 }
 
 static int read_lim_integration_time(void *user_data, harp_array data)
@@ -676,9 +625,8 @@ static int read_lim_sensor_altitude(void *user_data, harp_array data)
     return get_main_data((ingest_info *)user_data, "lim_ads", "alt", IS_NO_ARRAY, data.double_data);
 }
 
-static int read_lim_illumination_condition(void *user_data, long index, harp_array data)
+static int read_lim_illumination_condition(void *user_data, harp_array data)
 {
-    (void)index;        /* prevent unused warning */
     return get_illumination_condition((ingest_info *)user_data, "lim_summary_quality", data);
 }
 
@@ -750,6 +698,11 @@ static int lim_ingestion_init(const harp_ingestion_module *module, coda_product
     info->product = product;
     info->format_version = format_version;
     info->elements_per_profile = 0;
+    info->num_wavelengths = 0;
+    info->wavelengths = NULL;
+    info->sensitivity_curve_size = 0;
+    info->sensitivity_curve = NULL;
+    info->sensitivity_curve_wavelengths = NULL;
 
     info->upper = 1;
     if (harp_ingestion_options_has_option(options, "spectra"))
@@ -779,7 +732,16 @@ static int lim_ingestion_init(const harp_ingestion_module *module, coda_product
         return -1;
     }
     CHECKED_MALLOC(info->wavelengths, sizeof(double) * info->num_wavelengths);
-    get_wavelength_data(info, "lim_nom_wav_assignment", "nom_wl", info->wavelengths);
+    if (get_wavelength_data(info, "lim_nom_wav_assignment", "nom_wl", info->wavelengths) != 0)
+    {
+        ingestion_done(info);
+        return -1;
+    }
+    if (read_sensitivity_curve(info) != 0)
+    {
+        ingestion_done(info);
+        return -1;
+    }
 
     *definition = *module->product_definition;
     *user_data = info;
@@ -872,27 +834,40 @@ static void register_limb_product(void)
     description = "background spectral photon radiance of each spectrum measurement";
     variable_definition =
         harp_ingestion_register_variable_full_read(product_definition, "wavelength_photon_radiance", harp_type_double,
-                                                   2, dimension_type, NULL, description, "count/s/cm2/nm/sr", NULL,
+                                                   2, dimension_type, NULL, description, "count/s/cm2/nm/nsr", NULL,
                                                    read_lim_spectral_photon_radiance);
-    path = "/lim_mds[]/up_low_back_no_corr[0,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper and corrected=false", path, NULL);
-    path = "/lim_mds[]/up_low_back_no_corr[1,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower and corrected=false", path, NULL);
-    path = "/lim_mds[]/up_low_back_corr[0,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper and corrected=true", path, NULL);
-    path = "/lim_mds[]/up_low_back_corr[1,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower and corrected=true", path, NULL);
+    path = "/lim_mds[]/up_low_back_no_corr[0,], /lim_occultation_data[0]/abs_rad_sens_curve_limb[], "
+        "/lim_occultation_data[0]/rad_sens_curve_limb[], /lim_ads[]/off_back, /lim_ads[]/gain_back";
+    description = "radiance = (off_back + up_low_back_no_corr / gain_back) * "
+        "interp(abs_rad_sens_curve_limb, rad_sens_curve_limb, nom_wl)";
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper and corrected=false", path,
+                                         description);
+    path = "/lim_mds[]/up_low_back_no_corr[1,], /lim_occultation_data[0]/abs_rad_sens_curve_limb[], "
+        "/lim_occultation_data[0]/rad_sens_curve_limb[], /lim_ads[]/off_back, /lim_ads[]/gain_back";
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower and corrected=false", path,
+                                         description);
+    path = "/lim_mds[]/up_low_back_corr[0,], /lim_occultation_data[0]/abs_rad_sens_curve_limb[], "
+        "/lim_occultation_data[0]/rad_sens_curve_limb[], /lim_ads[]/off_back, /lim_ads[]/gain_back";
+    description = "radiance = (off_back + up_low_back_corr / gain_back) * "
+        "interp(abs_rad_sens_curve_limb, rad_sens_curve_limb, nom_wl)";
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper and corrected=true", path,
+                                         description);
+    path = "/lim_mds[]/up_low_back_corr[1,], /lim_occultation_data[0]/abs_rad_sens_curve_limb[], "
+        "/lim_occultation_data[0]/rad_sens_curve_limb[], /lim_ads[]/off_back, /lim_ads[]/gain_back";
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower and corrected=true", path,
+                                         description);
 
     /* wavelength_photon_radiance_uncertainty */
     description = "error in the background spectral photon radiance of each spectrum measurement";
     variable_definition =
         harp_ingestion_register_variable_full_read(product_definition, "wavelength_photon_radiance_uncertainty",
                                                    harp_type_double, 2, dimension_type, NULL, description,
-                                                   "count/s/cm2/nm/sr", NULL, read_lim_spectral_photon_radiance_error);
+                                                   "count/s/cm2/nm/nsr", NULL, read_lim_spectral_photon_radiance_error);
     path = "/lim_mds[]/err_up_low_back_corr[0,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper", path, NULL);
+    description = "uncertainty = (err_up_low_back_corr / 100) * wavelength_photon_radiance";
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=upper", path, description);
     path = "/lim_mds[]/err_up_low_back_corr[1,]";
-    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower", path, NULL);
+    harp_variable_definition_add_mapping(variable_definition, NULL, "spectra=lower", path, description);
 
     /* wavelength */
     description = "nominal wavelength assignment for each of the detector pixels";
@@ -935,9 +910,9 @@ static void register_limb_product(void)
     /* scene_type */
     description = "illumination condition for each profile";
     variable_definition =
-        harp_ingestion_register_variable_block_read(product_definition, "scene_type", harp_type_int8, 0,
-                                                    dimension_type, NULL, description, NULL, NULL,
-                                                    read_lim_illumination_condition);
+        harp_ingestion_register_variable_full_read(product_definition, "scene_type", harp_type_int8, 0,
+                                                   dimension_type, NULL, description, NULL, NULL,
+                                                   read_lim_illumination_condition);
     harp_variable_definition_set_enumeration_values(variable_definition, 5, scene_type_values);
     path = "/lim_summary_quality[0]/limb_flag";
     harp_variable_definition_add_mapping(variable_definition, NULL, "CODA product version 0", path, NULL);
@@ -1004,10 +979,7 @@ static int read_tra_wavelength_photon_transmittance_error(void *user_data, harp_
 
 static int read_tra_wavelength(void *user_data, harp_array data)
 {
-    int retval;
-
-    retval = get_wavelength_data((ingest_info *)user_data, "tra_nom_wav_assignment", "nom_wl", data.double_data);
-    return retval;
+    return get_wavelength_data((ingest_info *)user_data, "tra_nom_wav_assignment", "nom_wl", data.double_data);
 }
 
 static int read_tra_integration_time(void *user_data, harp_array data)
@@ -1030,9 +1002,8 @@ static int read_tra_sensor_altitude(void *user_data, harp_array data)
     return get_main_data((ingest_info *)user_data, "tra_geolocation", "alt", USE_ARRAY_INDEX_1, data.double_data);
 }
 
-static int read_tra_illumination_condition(void *user_data, long index, harp_array data)
+static int read_tra_illumination_condition(void *user_data, harp_array data)
 {
-    (void)index;        /* prevent unused warning */
     return get_illumination_condition((ingest_info *)user_data, "tra_summary_quality", data);
 }
 
@@ -1105,6 +1076,11 @@ static int tra_ingestion_init(const harp_ingestion_module *module, coda_product
     info->product = product;
     info->format_version = format_version;
     info->elements_per_profile = 0;
+    info->num_wavelengths = 0;
+    info->wavelengths = NULL;
+    info->sensitivity_curve_size = 0;
+    info->sensitivity_curve = NULL;
+    info->sensitivity_curve_wavelengths = NULL;
 
     if (tra_init_dimensions(info) != 0)
     {
@@ -1260,9 +1236,9 @@ static void register_tra_product(void)
     /* scene_type */
     description = "illumination condition for each profile";
     variable_definition =
-        harp_ingestion_register_variable_block_read(product_definition, "scene_type", harp_type_int8, 0,
-                                                    dimension_type, NULL, description, NULL, NULL,
-                                                    read_tra_illumination_condition);
+        harp_ingestion_register_variable_full_read(product_definition, "scene_type", harp_type_int8, 0,
+                                                   dimension_type, NULL, description, NULL, NULL,
+                                                   read_tra_illumination_condition);
     harp_variable_definition_set_enumeration_values(variable_definition, 5, scene_type_values);
     path = "/tra_summary_quality[0]/limb_flag";
     harp_variable_definition_add_mapping(variable_definition, NULL, "CODA product version 0", path, NULL);
