@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2019 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2020 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -822,54 +822,6 @@ LIBHARP_API double harp_mininf(void)
     return mkInf.as_double;
 }
 
-/** Write 64 bit signed integer to a string.
- * The string \a s will be 0 terminated.
- * \param a  A signed 64 bit integer value.
- * \param s  A character buffer that is at least 21 bytes long.
- */
-LIBHARP_API void harp_str64(int64_t a, char *s)
-{
-    if (a < 0)
-    {
-        s[0] = '-';
-        harp_str64u((uint64_t)(-a), &s[1]);
-    }
-    else
-    {
-        harp_str64u((uint64_t)a, s);
-    }
-}
-
-/** Write 64 bit unsigned integer to a string.
- * The string \a s will be 0 terminated.
- * \param a  An unsigned 64 bit integer value.
- * \param s  A character buffer that is at least 21 bytes long.
- */
-LIBHARP_API void harp_str64u(uint64_t a, char *s)
-{
-    if (a <= 4294967295UL)
-    {
-        sprintf(s, "%ld", (long)a);
-    }
-    else
-    {
-        long a1, a2;
-
-        a1 = (long)(a % 100000000);
-        a /= 100000000;
-        a2 = (long)(a % 100000000);
-        a /= 100000000;
-        if (a != 0)
-        {
-            sprintf(s, "%ld%08ld%08ld", (long)a, a2, a1);
-        }
-        else
-        {
-            sprintf(s, "%ld%08ld", a2, a1);
-        }
-    }
-}
-
 /**
  * @}
  */
@@ -983,6 +935,7 @@ long harp_parse_double(const char *buffer, long buffer_length, double *dst, int
     if (length > 0 && (*buffer == 'd' || *buffer == 'D' || *buffer == 'e' || *buffer == 'E'))
     {
         long exponent_value;
+        int exponent_overflow = 0;
 
         buffer++;
         length--;
@@ -1004,8 +957,16 @@ long harp_parse_double(const char *buffer, long buffer_length, double *dst, int
             {
                 break;
             }
-            exponent_value = 10 * exponent_value + (*buffer - '0');
-            exponent_length++;
+            if (!exponent_overflow)
+            {
+                exponent_value = 10 * exponent_value + (*buffer - '0');
+                exponent_length++;
+                if (exponent_value > 1024)
+                {
+                    /* we will now get 'inf' anyway, so stop parsing to prevent integer overflow in exponent */
+                    exponent_overflow = 1;
+                }
+            }
             buffer++;
             length--;
         }
