@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2019 S[&]T, The Netherlands.
+# Copyright (C) 2015-2020 S[&]T, The Netherlands.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -49,10 +49,12 @@ __all__ = ["Error", "CLibraryError", "UnsupportedTypeError", "UnsupportedDimensi
            "get_encoding", "set_encoding", "version", "import_product", "export_product", "concatenate",
            "execute_operations", "convert_unit", "to_dict"]
 
+
 class Error(Exception):
     """Exception base class for all HARP Python interface errors."""
     pass
 
+
 class CLibraryError(Error):
     """Exception raised when an error occurs inside the HARP C library.
 
@@ -77,6 +79,7 @@ class CLibraryError(Error):
     def __str__(self):
         return self.strerror
 
+
 class UnsupportedTypeError(Error):
     """Exception raised when unsupported types are encountered, either on the Python
     or on the C side of the interface.
@@ -84,6 +87,7 @@ class UnsupportedTypeError(Error):
     """
     pass
 
+
 class UnsupportedDimensionError(Error):
     """Exception raised when unsupported dimensions are encountered, either on the
     Python or on the C side of the interface.
@@ -91,6 +95,7 @@ class UnsupportedDimensionError(Error):
     """
     pass
 
+
 class NoDataError(Error):
     """Exception raised when the product returned from an import contains no variables,
     or variables without data.
@@ -99,6 +104,7 @@ class NoDataError(Error):
     def __init__(self):
         super(NoDataError, self).__init__("product contains no variables, or variables without data")
 
+
 class Variable(object):
     """Python representation of a HARP variable.
 
@@ -193,6 +199,7 @@ class Variable(object):
 
         return stream.getvalue()
 
+
 class Product(object):
     """Python representation of a HARP product.
 
@@ -336,6 +343,7 @@ class Product(object):
 
         return stream.getvalue()
 
+
 def _get_c_library_filename():
     """Return the filename of the HARP shared library depending on the current
     platform.
@@ -361,6 +369,7 @@ def _get_c_library_filename():
     # otherwise assume the library to be in the parent library directory
     return os.path.normpath(os.path.join(os.path.dirname(__file__), "../../..", library_name))
 
+
 def _get_filesystem_encoding():
     """Return the encoding used by the filesystem."""
     from sys import getdefaultencoding as _getdefaultencoding, getfilesystemencoding as _getfilesystemencoding
@@ -371,6 +380,7 @@ def _get_filesystem_encoding():
 
     return encoding
 
+
 def _init():
     """Initialize the HARP Python interface."""
     global _lib, _encoding, _py_dimension_type, _c_dimension_type, _py_data_type, _c_data_type_name
@@ -449,6 +459,7 @@ def _init():
             _lib.harp_type_string: "string"
         }
 
+
 def _dict_iteritems(dictionary):
     """Get an iterator or view on the items of the specified dictionary.
 
@@ -459,6 +470,7 @@ def _dict_iteritems(dictionary):
     except AttributeError:
         return dictionary.items()
 
+
 def _get_py_dimension_type(dimension_type):
     """Return the dimension name corresponding to the specified C dimension type
     code.
@@ -469,6 +481,7 @@ def _get_py_dimension_type(dimension_type):
     except KeyError:
         raise UnsupportedDimensionError("unsupported C dimension type code '%d'" % dimension_type)
 
+
 def _get_c_dimension_type(dimension_type):
     """Return the C dimension type code corresponding to the specified dimension
     name.
@@ -479,6 +492,7 @@ def _get_c_dimension_type(dimension_type):
     except KeyError:
         raise UnsupportedDimensionError("unsupported dimension %r" % dimension_type)
 
+
 def _get_py_data_type(data_type):
     """Return the Python type corresponding to the specified C data type code."""
     try:
@@ -486,6 +500,7 @@ def _get_py_data_type(data_type):
     except KeyError:
         raise UnsupportedTypeError("unsupported C data type code '%d'" % data_type)
 
+
 def _get_c_data_type(value):
     """Return the C data type code corresponding to the specified variable data
     value.
@@ -534,6 +549,7 @@ def _get_c_data_type(value):
     else:
         raise UnsupportedTypeError("unsupported type %r" % value.__class__.__name__)
 
+
 def _get_c_data_type_name(data_type):
     """Return the canonical name for the specified C data type code."""
     try:
@@ -541,6 +557,7 @@ def _get_c_data_type_name(data_type):
     except KeyError:
         raise UnsupportedTypeError("unsupported C data type code '%d'" % data_type)
 
+
 def _c_can_cast(c_data_type_src, c_data_type_dst):
     """Returns True if the source C data type can be cast to the destination C data
     type while preserving values.
@@ -571,6 +588,7 @@ def _c_can_cast(c_data_type_src, c_data_type_dst):
     else:
         return False
 
+
 def _encode_string_with_encoding(string, encoding="utf-8"):
     """Encode a unicode string using the specified encoding.
 
@@ -598,6 +616,7 @@ def _encode_string_with_encoding(string, encoding="utf-8"):
     except UnicodeEncodeError:
         raise Error("cannot encode '%s' using encoding '%s'" % (string, encoding))
 
+
 def _decode_string_with_encoding(string, encoding="utf-8"):
     """Decode a byte string using the specified encoding.
 
@@ -628,6 +647,7 @@ def _decode_string_with_encoding(string, encoding="utf-8"):
     except UnicodeEncodeError:
         raise Error("cannot decode '%s' using encoding '%s'" % (string, encoding))
 
+
 def _encode_path(path):
     """Encode the input unicode path using the filesystem encoding.
 
@@ -645,6 +665,7 @@ def _encode_path(path):
     else:
         raise TypeError("path must be bytes or str, not %r" % path.__class__.__name__)
 
+
 def _encode_string(string):
     """Encode the input unicode string using the package default encoding.
 
@@ -662,6 +683,7 @@ def _encode_string(string):
     else:
         raise TypeError("string must be bytes or str, not %r" % string.__class__.__name__)
 
+
 def _decode_string(string):
     """Decode the input byte string using the package default encoding.
 
@@ -678,6 +700,7 @@ def _decode_string(string):
     else:
         raise TypeError("string must be bytes or str, not %r" % string.__class__.__name__)
 
+
 def _format_data_type(data):
     """Return the string representation of the C data type that would be used to
     store the specified data, or "<invalid>" if the specified data is of an
@@ -689,6 +712,7 @@ def _format_data_type(data):
     except UnsupportedTypeError:
         return "<invalid>"
 
+
 def _format_dimensions(dimension, data):
     """Construct a formatted string from the specified dimensions and data that
     provides information about dimension types and lengths, or "<invalid>" if this
@@ -712,6 +736,7 @@ def _format_dimensions(dimension, data):
 
     return stream.getvalue()
 
+
 def _import_scalar(c_data_type, c_data):
     if c_data_type == _lib.harp_type_int8:
         return c_data.int8_data
@@ -726,6 +751,7 @@ def _import_scalar(c_data_type, c_data):
 
     raise UnsupportedTypeError("unsupported C data type code '%d'" % c_data_type)
 
+
 def _import_array(c_data_type, c_num_elements, c_data):
     if c_data_type == _lib.harp_type_string:
         data = numpy.empty((c_num_elements,), dtype=numpy.object)
@@ -739,6 +765,7 @@ def _import_array(c_data_type, c_num_elements, c_data):
     c_data_buffer = _ffi.buffer(c_data.ptr, c_num_elements * _lib.harp_get_size_for_type(c_data_type))
     return numpy.copy(numpy.frombuffer(c_data_buffer, dtype=_get_py_data_type(c_data_type)))
 
+
 def _import_variable(c_variable):
     # Import variable data.
     data = _import_array(c_variable.data_type, c_variable.num_elements, c_variable.data)
@@ -768,6 +795,7 @@ def _import_variable(c_variable):
 
     return variable
 
+
 def _import_product(c_product):
     product = Product()
 
@@ -786,6 +814,7 @@ def _import_product(c_product):
 
     return product
 
+
 def _export_scalar(data, c_data_type, c_data):
     if c_data_type == _lib.harp_type_int8:
         c_data.int8_data = data
@@ -800,6 +829,7 @@ def _export_scalar(data, c_data_type, c_data):
     else:
         raise UnsupportedTypeError("unsupported C data type code '%d'" % c_data_type)
 
+
 def _export_array(data, c_variable):
     if c_variable.data_type != _lib.harp_type_string:
         # NB. The _ffi.buffer() method as well as the numpy.frombuffer() method provide a view on the C array; neither
@@ -820,6 +850,7 @@ def _export_array(data, c_variable):
         if _lib.harp_variable_set_string_data_element(c_variable, 0, _encode_string(data)) != 0:
             raise CLibraryError()
 
+
 def _export_variable(name, variable, c_product):
     data = getattr(variable, "data", None)
     if data is None:
@@ -869,9 +900,9 @@ def _export_variable(name, variable, c_product):
         _lib.harp_variable_delete(c_variable_ptr[0])
         raise CLibraryError()
 
-    # The C variable has been successfully added to the C product. Therefore, the memory management of the C variable is
-    # tied to the life time of the C product. If an error occurs, the memory occupied by the C variable will be freed
-    # along with the C product.
+    # The C variable has been successfully added to the C product. Therefore, the memory management of the C variable
+    # is tied to the life time of the C product. If an error occurs, the memory occupied by the C variable will be
+    # freed along with the C product.
     c_variable = c_variable_ptr[0]
 
     # Copy data into the C variable.
@@ -937,9 +968,11 @@ def _export_variable(name, variable, c_product):
         pass
     else:
         if enum and _lib.harp_variable_set_enumeration_values(c_variable, len(enum),
-                                                              [_ffi.new("char[]", _encode_string(name)) for name in enum]) != 0:
+                                                              [_ffi.new("char[]",
+                                                               _encode_string(name)) for name in enum]) != 0:
             raise CLibraryError()
 
+
 def _export_product(product, c_product):
     # Export product attributes.
     try:
@@ -965,6 +998,7 @@ def _export_product(product, c_product):
         except Error as _error:
             raise Error("variable '%r' could not be exported (%s)" % (name, str(_error)))
 
+
 def _update_history(product, command):
     line = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
     line += " [harp-%s] " % (version())
@@ -974,6 +1008,7 @@ def _update_history(product, command):
     except AttributeError:
         product.history = line
 
+
 def get_encoding():
     """Return the encoding used to convert between unicode strings and C strings
     (only relevant when using Python 3).
@@ -981,6 +1016,7 @@ def get_encoding():
     """
     return _encoding
 
+
 def set_encoding(encoding):
     """Set the encoding used to convert between unicode strings and C strings
     (only relevant when using Python 3).
@@ -990,10 +1026,12 @@ def set_encoding(encoding):
 
     _encoding = encoding
 
+
 def version():
     """Return the version of the HARP C library."""
     return _decode_string(_ffi.string(_lib.libharp_version))
 
+
 def to_dict(product):
     """Convert a Product to an OrderedDict.
 
@@ -1026,7 +1064,8 @@ def to_dict(product):
 
     return dictionary
 
-def import_product(filename, operations="", options="", post_operations=""):
+
+def import_product(filename, operations="", options="", reduce_operations="", post_operations=""):
     """Import a product from a file.
 
     This will first try to import the file as an HDF4, HDF5, or netCDF file that
@@ -1049,6 +1088,13 @@ def import_product(filename, operations="", options="", post_operations=""):
     options -- Ingestion module specific options; should be specified as a semi-
                colon separated string of key=value pairs; only used if a file is not
                in HARP format.
+    reduce_operations -- Actions to apply after each append; should be specified as a
+                       semi-colon separated string of operations;
+                       these operations will only be applied if the filename argument is
+                       a file pattern or a list of filenames/patterns;
+                       this advanced option allows for memory efficient application
+                       of time reduction operations (such as bin()) that would
+                       normally be provided as part of post_operations.
     post_operations -- Actions to apply after the list of products is merged; should be
                        specified as a semi-colon separated string of operations;
                        these operations will only be applied if the filename argument is
@@ -1083,17 +1129,23 @@ def import_product(filename, operations="", options="", post_operations=""):
                     raise CLibraryError()
                 if _lib.harp_product_is_empty(c_product_ptr[0]) == 1:
                     _lib.harp_product_delete(c_product_ptr[0])
-                elif merged_product_ptr is None:
-                    merged_product_ptr = c_product_ptr
-                    # if this remains the only product then make sure it still looks like it was the result of a merge
-                    if _lib.harp_product_append(merged_product_ptr[0], _ffi.NULL) != 0:
-                        raise CLibraryError()
                 else:
-                    try:
-                        if _lib.harp_product_append(merged_product_ptr[0], c_product_ptr[0]) != 0:
+                    if merged_product_ptr is None:
+                        merged_product_ptr = c_product_ptr
+                        # if this remains the only product, make sure it still looks like it was the result of a merge
+                        if _lib.harp_product_append(merged_product_ptr[0], _ffi.NULL) != 0:
+                            raise CLibraryError()
+                    else:
+                        try:
+                            if _lib.harp_product_append(merged_product_ptr[0], c_product_ptr[0]) != 0:
+                                raise CLibraryError()
+                        finally:
+                            _lib.harp_product_delete(c_product_ptr[0])
+                    if reduce_operations:
+                        # perform reduction operations on the partially merged product after each append
+                        if _lib.harp_product_execute_operations(merged_product_ptr[0],
+                                                                _encode_string(reduce_operations)) != 0:
                             raise CLibraryError()
-                    finally:
-                        _lib.harp_product_delete(c_product_ptr[0])
         except:
             if merged_product_ptr is not None:
                 _lib.harp_product_delete(merged_product_ptr[0])
@@ -1118,6 +1170,8 @@ def import_product(filename, operations="", options="", post_operations=""):
                 command += ",operations='{0}'".format(operations)
             if options:
                 command += ",options='{0}'".format(options)
+            if reduce_operations:
+                command += ",reduce_operations='{0}'".format(reduce_operations)
             if post_operations:
                 command += ",post_operations='{0}'".format(post_operations)
             command += ")"
@@ -1125,7 +1179,6 @@ def import_product(filename, operations="", options="", post_operations=""):
 
         return product
 
-
     c_product_ptr = _ffi.new("harp_product **")
 
     # Import the product as a C product.
@@ -1156,6 +1209,7 @@ def import_product(filename, operations="", options="", post_operations=""):
     finally:
         _lib.harp_product_delete(c_product_ptr[0])
 
+
 def export_product(product, filename, file_format="netcdf", operations="", hdf5_compression=0):
     """Export a HARP compliant product.
 
@@ -1248,11 +1302,11 @@ def concatenate(products):
     variable_names = []
     for product in products:
         for name in product:
-            if not name in variable_names and name != "index":
+            if name not in variable_names and name != "index":
                 variable_names.append(name)
     for name in variable_names:
         for product in products:
-            if not name in product:
+            if name not in product:
                 raise Error("not all products contain variable '%s'" % (name,))
 
     for product in products:
@@ -1261,7 +1315,7 @@ def concatenate(products):
     for name in variable_names:
         for product in products:
             source_variable = product[name]
-            if not name in target_product:
+            if name not in target_product:
                 target_variable = Variable(source_variable.data, source_variable.dimension)
                 if hasattr(source_variable, 'unit'):
                     target_variable.unit = source_variable.unit
@@ -1321,6 +1375,7 @@ def execute_operations(products, operations="", post_operations=""):
                         raise CLibraryError()
                 except:
                     _lib.harp_product_delete(c_product_ptr[0])
+                    raise
 
                 if _lib.harp_product_is_empty(c_product_ptr[0]) == 1:
                     _lib.harp_product_delete(c_product_ptr[0])
