@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2019 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2020 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@
 #include "harp.h"
 
 #include <assert.h>
+#include <locale.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -1449,6 +1450,9 @@ int matchup(int argc, char *argv[])
     collocation_info *info = NULL;
     int i;
 
+    /* set locale to a default one so our use of strtod() works as intended */
+    setlocale(LC_NUMERIC, "C");
+
     if (collocation_info_new(&info) != 0)
     {
         return -1;
